@@ -208,8 +208,7 @@ class IntColumn(column.Column):
         return converted
 
     def _create_array(self, size=0):
-        val = self.get_default_value()
-        return np.array([val] * size, dtype=np.int32)
+        return np.zeros(size, dtype=np.int32)
 
 class NullableIntColumn(column.Column):
     """A Column holding nullable int values.
