@@ -209,8 +209,7 @@ class LongColumn(column.Column):
         return converted
 
     def _create_array(self, size=0):
-        val = self.get_default_value()
-        return np.array([val] * size, dtype=np.int64)
+        return np.zeros(size, dtype=np.int64)
 
 class NullableLongColumn(column.Column):
     """A Column holding nullable long values.
