@@ -207,8 +207,7 @@ class ShortColumn(column.Column):
         return converted
 
     def _create_array(self, size=0):
-        val = self.get_default_value()
-        return np.array([val] * size, dtype=np.int16)
+        return np.zeros(size, dtype=np.int16)
 
 class NullableShortColumn(column.Column):
     """A Column holding nullable short values.
